﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Media;

namespace AsyncLibrary
{
    public class Downloader
    {
        public Task<LinkInfo> DownloadItemAsync(string uri)
        {
            return DownloadItemAsync(uri, CancellationToken.None);
        }

        public Task<LinkInfo> DownloadItemAsync(string uri, CancellationToken cancellationToken)
        {
            if (string.IsNullOrEmpty(uri) == true)
                throw new ArgumentException("uri");

            return DownloadItemAsyncInternal(uri, cancellationToken);
        }

        private async Task<LinkInfo> DownloadItemAsyncInternal(string uri, CancellationToken cancellationToken)
        {
            string item = await new WebClient().DownloadStringTaskAsync(uri).ConfigureAwait( false );
            LinkInfo linkInfo = await Task.Run(() => new LinkInfo { Length = item.Length, Html = item, Title = GetTitle(item) });
            PollItemAsync(uri, linkInfo, cancellationToken);
            return linkInfo;
        }

        private async void PollItemAsync(string uri, LinkInfo link, CancellationToken cancellationToken)
        {
            Random r = new Random();

            try
            {
                while (true)
                {
                    cancellationToken.ThrowIfCancellationRequested();
                    await Task.Delay(5000, cancellationToken);

                    cancellationToken.ThrowIfCancellationRequested();
                    string item = await new WebClient().DownloadStringTaskAsync(new Uri(uri)).ConfigureAwait(false);
                    if (item.Length != link.Length)
                    {
                        link.Title = GetTitle(item);
                        link.Length = item.Length;
                        link.Html = item;
                        link.Color = Color.FromArgb(0xff, (byte)r.Next(255), (byte)r.Next(255), (byte)r.Next(255));
                    }
                }
            }
            catch (OperationCanceledException)
            {
            }
        }


        public string GetTitle(string html)
        {
            Match m = Regex.Match(html, @"(?<=<title.*>)([\s\S]*)(?=</title>)", RegexOptions.IgnoreCase);
            return m.Value;
        }

    }
}
