﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Project_4
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private CancellationTokenSource _cancellationToken;
        public MainWindow()
        {
            InitializeComponent();
        }

        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            _cancellationToken = new CancellationTokenSource();
            ObservableCollection<string> uris = new ObservableCollection<string>();
            ObservableCollection<LinkInfo> linkInfos = new ObservableCollection<LinkInfo>();

            btnDoWork.IsEnabled = false;

            txtHtml.Text = "";
            lstLinks.ItemsSource = null;

            try
            {
                string result = await new WebClient().DownloadStringTaskAsync("http://msdn.microsoft.com/");
                _cancellationToken.Token.ThrowIfCancellationRequested();
                txtHtml.Text = result;

                await Task.Run(() =>
                {
                    MatchCollection mc = Regex.Matches(result, "href\\s*=\\s*(?:\"(?<1>http://[^\"]*)\")", RegexOptions.IgnoreCase);
                    foreach (Match m in mc)
                    {
                        uris.Add(m.Groups[1].Value);
                    }
                });
                lstLinks.ItemsSource = linkInfos;
                LinkInfo[] links = await Task.WhenAll(from uri in uris select DownloadItem(uri, _cancellationToken.Token));
                foreach (LinkInfo link in links)
                {
                    linkInfos.Add(link);
                }

            }
            catch (OperationCanceledException)
            {
                MessageBox.Show("Vorgang abgebrochen");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }

            btnDoWork.IsEnabled = true;
        }

        public async Task<LinkInfo> DownloadItem(string uri, CancellationToken cancellationToken)
        {
            string item = await new WebClient().DownloadStringTaskAsync(uri );
            LinkInfo linkInfo = await Task.Run(() => new LinkInfo { Length = item.Length, Html = item, Title = GetTitle(item) });
            PollItem(uri, linkInfo, cancellationToken);
            return linkInfo;
        }


        private async void PollItem(string uri, LinkInfo link, CancellationToken cancellationToken)
        {
            Random r = new Random();

            try
            {
                while (true)
                {
                    cancellationToken.ThrowIfCancellationRequested();
                    await Task.Delay(5000, cancellationToken);

                    cancellationToken.ThrowIfCancellationRequested();
                    string item = await new WebClient().DownloadStringTaskAsync(new Uri(uri));
                    if (item.Length != link.Length)
                    {
                        link.Title = GetTitle(item);
                        link.Length = item.Length;
                        link.Html = item;
                        link.Color = Color.FromArgb(0xff, (byte)r.Next(255), (byte)r.Next(255), (byte)r.Next(255));
                    }
                }
            }
            catch (OperationCanceledException)
            {
            }
        }


        public string GetTitle(string html)
        {
            Match m = Regex.Match(html, @"(?<=<title.*>)([\s\S]*)(?=</title>)", RegexOptions.IgnoreCase);
            return m.Value;
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            _cancellationToken.Cancel();
        }
    }
}
